import os
import json
import logging
import re
from datetime import datetime
from openai import OpenAI


def extract_json_from_text(text):
    """
    从文本中提取JSON部分
    
    Args:
        text (str): 包含JSON的文本
        
    Returns:
        str: 提取的JSON字符串
    """
    # 尝试找到最外层的花括号对
    start_idx = text.find('{')
    if start_idx == -1:
        return "{}"
    
    # 使用简单的括号匹配算法找到对应的闭合括号
    count = 0
    for i in range(start_idx, len(text)):
        if text[i] == '{':
            count += 1
        elif text[i] == '}':
            count -= 1
            if count == 0:
                return text[start_idx:i+1]
    
    # 如果没有找到闭合括号，返回从开始位置到文本结尾
    return text[start_idx:]


def process_webpage_with_QWen(markdown_text, publish_time):
    """
    使用阿里云的 Qwen VL Max 模型解析网页 markdown 文本中的名片信息
    
    Args:
        markdown_text (str): 网页的 markdown 格式文本内容
        publish_time (str): 发布时间，用于career_path中的date字段
        
    Returns:
        dict: 解析的名片信息
    """
    # 阿里云 Qwen API 配置
    QWEN_API_KEY = os.environ.get('QWEN_API_KEY', 'sk-8f2320dafc9e4076968accdd8eebd8e9')
    
    try:
        # 初始化 OpenAI 客户端，配置为阿里云 API
        client = OpenAI(
            api_key=QWEN_API_KEY,
            base_url="https://dashscope.aliyuncs.com/compatible-mode/v1",
        )
        
        # 构建针对网页文本的优化提示语
        prompt = """你是酒店行业人事任命信息提取专家。请仔细分析提供的网页Markdown文本内容，精确提取其中的人员任命信息。

## 重要说明
1. **多人员处理**: 文本中可能包含多个人的任命信息，通常以数字编号（如**1**、**2**、**3**等）分隔不同人员。请将所有人员信息都提取出来，以JSON数组格式返回。请仔细查看整个文档，不要遗漏任何人员信息。
2. **照片链接识别**: 人物照片链接通常出现在人物姓名的前面，通过空行分隔。请优先关联距离最近的照片链接。
3. **字段限制**: 只需要提取指定的9个字段，其他信息忽略。
4. **完整性要求**: 请确保提取文档中所有被任命的人员信息，包括总经理、副总裁、总监等各级管理人员。每个数字编号下的人员都需要提取。

## 提取要求
- 区分中英文内容，分别提取
- 保持提取信息的原始格式（如大小写、标点）
- 对于无法识别或文本中不存在的信息，返回空字符串
- 文本中没有的信息，请不要猜测
- 从酒店任命公告中识别人员基本信息和职位信息

## 照片链接识别规则
- 照片通常以 ![描述](URL) 格式出现
- 照片链接通常位于人物姓名的前面，通过空行分隔
- 每个人物对应其前面最近的一张照片
- 如果照片无法明确对应某个人物，则该人物的pic_url为空字符串

## 需提取的字段（仅这9个字段）
1. 中文姓名 (name_zh) - 人物的中文姓名
2. 英文姓名 (name_en) - 人物的英文姓名，如果没有则为空字符串
3. 中文职位/头衔 (title_zh) - 人物的中文职位或头衔
4. 英文职位/头衔 (title_en) - 人物的英文职位或头衔，如果没有则为空字符串
5. 中文酒店/公司名称 (hotel_zh) - 人物所在的中文酒店或公司名称
6. 英文酒店/公司名称 (hotel_en) - 人物所在的英文酒店或公司名称，如果没有则为空字符串
7. 品牌组合 (brand_group) - 酒店所属的品牌集团，如万豪、希尔顿等
8. 照片链接 (pic_url) - 人物的照片URL链接，根据上述识别规则提取

## 输出格式
请以严格的JSON格式返回结果，不要添加任何额外解释文字。如果文本中包含多个人员信息，返回JSON数组，每个人员一个JSON对象。如果只有一个人员，也要返回数组格式。

单人员示例：
```json
[
  {
    "name_zh": "张三",
    "name_en": "Zhang San",
    "title_zh": "总经理",
    "title_en": "General Manager",
    "hotel_zh": "北京万豪酒店",
    "hotel_en": "Beijing Marriott Hotel",
    "brand_group": "万豪",
    "pic_url": "https://example.com/photo1.jpg"
  }
]
```

多人员示例：
```json
[
  {
    "name_zh": "张三",
    "name_en": "Zhang San",
    "title_zh": "总经理",
    "title_en": "General Manager",
    "hotel_zh": "北京万豪酒店",
    "hotel_en": "Beijing Marriott Hotel",
    "brand_group": "万豪",
    "pic_url": "https://example.com/photo1.jpg"
  },
  {
    "name_zh": "李四",
    "name_en": "Li Si",
    "title_zh": "市场总监",
    "title_en": "Marketing Director",
    "hotel_zh": "上海希尔顿酒店",
    "hotel_en": "Shanghai Hilton Hotel",
    "brand_group": "希尔顿",
    "pic_url": "https://example.com/photo2.jpg"
  }
]
```

发现有数字编号（**1**、**2**、**3**）分隔，应该提取三个人员的信息，返回包含3个对象的数组。

## 特别提醒
- 请务必扫描整个文档，查找所有数字编号标记的人员信息
- 每个人员信息都要单独提取成一个JSON对象
- 最终返回的数组长度应该等于文档中的人员数量

以下是需要分析的网页Markdown文本内容：

""" + markdown_text
        
        # 调用 Qwen VL Max API
        logging.info("发送网页文本请求到 Qwen VL Max 模型")
        completion = client.chat.completions.create(
            model="qwen-vl-max-latest",
            messages=[
                {
                    "role": "user",
                    "content": [
                        {"type": "text", "text": prompt}
                    ]
                }
            ],
            temperature=0.1,  # 降低温度增加精确性
            response_format={"type": "json_object"}  # 要求输出JSON格式
        )
        
        # 解析响应
        response_content = completion.choices[0].message.content
        logging.info(f"成功从 Qwen 模型获取网页文本响应: {response_content}")
        
        # 直接解析 QWen 返回的 JSON 响应
        try:
            extracted_data = json.loads(response_content)
            logging.info("成功解析 Qwen 网页文本响应中的 JSON")
        except json.JSONDecodeError as e:
            error_msg = f"JSON 解析失败: {str(e)}"
            logging.error(error_msg)
            raise Exception(error_msg)

        # 确保返回的是数组格式
        if not isinstance(extracted_data, list):
            # 如果返回的不是数组，包装成数组
            extracted_data = [extracted_data] if extracted_data else []
        
        # 确保数组中每个人员对象都包含所有必要字段
        required_fields = [
            'name_zh', 'name_en', 'title_zh', 'title_en', 
            'hotel_zh', 'hotel_en', 'brand_group', 'pic_url'
        ]
        
        for person in extracted_data:
            for field in required_fields:
                if field not in person:
                    person[field] = ""
            
            # 为每个人员添加career_path字段
            career_entry = {
                'date': publish_time,
                'hotel_en': person.get('hotel_en', ''),
                'hotel_zh': person.get('hotel_zh', ''),
                'image_path': '',
                'source': 'webpage_extraction',
                'title_en': person.get('title_en', ''),
                'title_zh': person.get('title_zh', '')
            }
            
            person['career_path'] = [career_entry]
            logging.info(f"为人员 {person.get('name_zh', 'Unknown')} 添加了career_path记录: {career_entry}")
        
        return extracted_data
        
    except Exception as e:
        error_msg = f"Qwen VL Max 模型网页文本解析失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        raise Exception(error_msg) 